@@ -19,28 +19,36 @@
 
 #pragma once
 
+#include <ratelimiter.h>
+#include <unity/thumbnailer/qt/thumbnailer-qt.h>
+
 #include <QQuickImageProvider>
-#include <thumbnailerinterface.h>
 
 #include <memory>
 
 namespace unity
 {
+
 namespace thumbnailer
 {
+
 namespace qml
 {
 
 class AlbumArtGenerator : public QQuickAsyncImageProvider
 {
 private:
-    std::unique_ptr<QDBusConnection> connection;
-    std::unique_ptr<ThumbnailerInterface> iface;
+    std::shared_ptr<unity::thumbnailer::qt::Thumbnailer> thumbnailer;
+    std::shared_ptr<unity::thumbnailer::RateLimiter> backlog_limiter;
 
 public:
-    AlbumArtGenerator();
+    AlbumArtGenerator(std::shared_ptr<unity::thumbnailer::qt::Thumbnailer> thumbnailer,
+                      std::shared_ptr<unity::thumbnailer::RateLimiter> backlog_limiter);
     QQuickImageResponse* requestImageResponse(const QString& id, const QSize& requestedSize) override;
 };
-}
-}
-}
+
+}  // namespace qml
+
+}  // namespace thumbnailer
+
+}  // namespace unity
