@@ -20,8 +20,9 @@
 
 #include "artistartgenerator.h"
 
-#include "artgeneratorcommon.h"
-#include <service/dbus_names.h>
+#include <QDebug>
+#include <QUrlQuery>
+
 #include "thumbnailerimageresponse.h"
 
 namespace
@@ -40,8 +41,11 @@ namespace thumbnailer
 namespace qml
 {
 
-ArtistArtGenerator::ArtistArtGenerator()
+ArtistArtGenerator::ArtistArtGenerator(std::shared_ptr<unity::thumbnailer::qt::Thumbnailer> thumbnailer,
+                                       std::shared_ptr<unity::thumbnailer::RateLimiter> backlog_limiter)
     : QQuickAsyncImageProvider()
+    , thumbnailer(thumbnailer)
+    , backlog_limiter(backlog_limiter)
 {
 }
 
@@ -53,8 +57,7 @@ QQuickImageResponse* ArtistArtGenerator::requestImageResponse(const QString& id,
     {
         qWarning().nospace() << "ArtistArtGenerator::requestImageResponse(): deprecated invalid QSize: "
                              << requestedSize << ". This feature will be removed soon. Pass the desired size instead.";
-        size.setWidth(128);
-        size.setHeight(128);
+        // Size will be adjusted by the service to 128x128.
     }
 
     QUrlQuery query(id);
@@ -64,22 +67,15 @@ QQuickImageResponse* ArtistArtGenerator::requestImageResponse(const QString& id,
         return new ThumbnailerImageResponse(requestedSize, DEFAULT_ARTIST_ART);
     }
 
-    if (!connection)
-    {
-        // Create connection here and not on the constructor, so it belongs to the proper thread.
-        connection.reset(new QDBusConnection(
-            QDBusConnection::connectToBus(QDBusConnection::SessionBus, "album_art_generator_dbus_connection")));
-        iface.reset(new ThumbnailerInterface(service::BUS_NAME, service::THUMBNAILER_BUS_PATH, *connection));
-    }
-
     const QString artist = query.queryItemValue("artist", QUrl::FullyDecoded);
     const QString album = query.queryItemValue("album", QUrl::FullyDecoded);
 
-    // perform dbus call
-    auto reply = iface->GetArtistArt(artist, album, size);
-    std::unique_ptr<QDBusPendingCallWatcher> watcher(
-        new QDBusPendingCallWatcher(reply));
-    return new ThumbnailerImageResponse(size, DEFAULT_ARTIST_ART, std::move(watcher));
+    // Schedule dbus call
+    auto job = [this, artist, album, size]
+    {
+        return thumbnailer->getArtistArt(artist, album, size);
+    };
+    return new ThumbnailerImageResponse(size, DEFAULT_ARTIST_ART, backlog_limiter.get(), job);
 }
 
 }  // namespace qml
