@@ -18,10 +18,13 @@
 
 #include "plugin.h"
 
+#include <settings.h>
 #include "albumartgenerator.h"
 #include "artistartgenerator.h"
 #include "thumbnailgenerator.h"
 
+#include <memory>
+
 namespace unity
 {
 
@@ -34,16 +37,18 @@ namespace qml
 void ThumbnailerPlugin::registerTypes(const char* uri)
 {
     qmlRegisterTypeNotAvailable(uri, 0, 1, "__ThumbnailerIgnoreMe",
-                                "Ignore this: QML plugins must contain at least one type");
+                                QStringLiteral("Ignore this: QML plugins must contain at least one type"));
 }
 
 void ThumbnailerPlugin::initializeEngine(QQmlEngine* engine, const char* uri)
 {
     QQmlExtensionPlugin::initializeEngine(engine, uri);
 
+    auto thumbnailer = std::make_shared<unity::thumbnailer::qt::Thumbnailer>();
+
     try
     {
-        engine->addImageProvider("albumart", new AlbumArtGenerator());
+        engine->addImageProvider("albumart", new AlbumArtGenerator(thumbnailer));
     }
     // LCOV_EXCL_START
     catch (const std::exception& e)
@@ -59,7 +64,7 @@ void ThumbnailerPlugin::initializeEngine(QQmlEngine* engine, const char* uri)
 
     try
     {
-        engine->addImageProvider("artistart", new ArtistArtGenerator());
+        engine->addImageProvider("artistart", new ArtistArtGenerator(thumbnailer));
     }
     // LCOV_EXCL_START
     catch (const std::exception& e)
@@ -75,7 +80,7 @@ void ThumbnailerPlugin::initializeEngine(QQmlEngine* engine, const char* uri)
 
     try
     {
-        engine->addImageProvider("thumbnailer", new ThumbnailGenerator());
+        engine->addImageProvider("thumbnailer", new ThumbnailGenerator(thumbnailer));
     }
     // LCOV_EXCL_START
     catch (const std::exception& e)
