@@ -20,7 +20,7 @@
 #include <internal/ubuntuserverdownloader.h>
 
 #include <internal/artreply.h>
-#include <internal/settings.h>
+#include <settings.h>
 
 #include <QNetworkReply>
 #include <QTimer>
@@ -81,6 +81,7 @@ bool network_down_error(QNetworkReply::NetworkError error)
     switch (error)
     {
         // add here all the cases that you consider as network errors
+        case QNetworkReply::ConnectionRefusedError:
         case QNetworkReply::HostNotFoundError:
         case QNetworkReply::OperationCanceledError:
         case QNetworkReply::TemporaryNetworkFailureError:
@@ -98,7 +99,14 @@ bool network_down_error(QNetworkReply::NetworkError error)
 
 class UbuntuServerArtReply : public ArtReply
 {
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Winconsistent-missing-override"
+#endif
     Q_OBJECT
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 
 public:
     Q_DISABLE_COPY(UbuntuServerArtReply)
@@ -107,7 +115,7 @@ public:
     UbuntuServerArtReply(QString const& url)
         : ArtReply(nullptr)
         , is_running_(false)
-        , error_string_("network down")
+        , error_string_(QStringLiteral("network down"))
         , error_(QNetworkReply::TemporaryNetworkFailureError)
         , url_string_(url)
         , succeeded_(false)
@@ -184,8 +192,10 @@ public Q_SLOTS:
         //       reply_ is nullptr only if the network is down.
         if (!reply_)
         {
-            Q_EMIT finished();  // LCOV_EXCL_LINE
+            // LCOV_EXCL_START
+            Q_EMIT finished();
             return;
+            // LCOV_EXCL_STOP
         }
 
         timer_.stop();
@@ -238,9 +248,9 @@ QUrl get_art_url(
     }
 
     QUrlQuery q;
-    q.addQueryItem("artist", artist);
-    q.addQueryItem("album", album);
-    q.addQueryItem("key", api_key);
+    q.addQueryItem(QStringLiteral("artist"), artist);
+    q.addQueryItem(QStringLiteral("album"), album);
+    q.addQueryItem(QStringLiteral("key"), api_key);
 
     QUrl url(prefix_api_root + "/" + base_url);
     url.setQuery(q);
