@@ -27,11 +27,19 @@
 #pragma GCC diagnostic ignored "-Wold-style-cast"
 #pragma GCC diagnostic ignored "-Wcast-qual"
 #pragma GCC diagnostic ignored "-Wcast-align"
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wparentheses-equality"
+#endif
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gst/gst.h>
 #include <gst/tag/tag.h>
 
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
+
 #include <internal/gobj_memory.h>
 
 namespace unity
