@@ -14,22 +14,18 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Authored by: Jussi Pakkanen <jussi.pakkanen@canonical.com>
+ *              Michi Henning <michi.henning@canonical.com>
  */
 
+#include "thumbnailextractor.h"
+
+#include <internal/trace.h>
+
+#include <boost/algorithm/string.hpp>
+
 #include <cstdio>
-#include <string>
 #include <memory>
 #include <stdexcept>
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wold-style-cast"
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#pragma GCC diagnostic ignored "-Wcast-align"
-#include <gst/gst.h>
-#include <gio/gio.h>
-#include <glib.h>
-#pragma GCC diagnostic pop
-
-#include "thumbnailextractor.h"
 
 using namespace std;
 using namespace unity::thumbnailer::internal;
@@ -40,15 +36,7 @@ namespace
 string command_line_arg_to_uri(string const& arg)
 {
     unique_ptr<GFile, decltype(&g_object_unref)> file(g_file_new_for_commandline_arg(arg.c_str()), g_object_unref);
-    if (!file)
-    {
-        throw runtime_error("Could not create parse argument as file");
-    }
     char* c_uri = g_file_get_uri(file.get());
-    if (!c_uri)
-    {
-        throw runtime_error("Could not convert to uri");
-    }
     string uri(c_uri);
     g_free(c_uri);
     return uri;
@@ -59,54 +47,53 @@ bool extract_thumbnail(string const& uri, string const& ofname)
     ThumbnailExtractor extractor;
 
     extractor.set_uri(uri);
-    if (extractor.has_video())
+    if (extractor.extract_cover_art())
     {
-        if (!extractor.extract_video_frame())
-        {
-            return false;
-        }
+        extractor.save_screenshot(ofname);
+        return true;
     }
-    else
+    if (extractor.has_video() && extractor.extract_video_frame())
     {
-        if (!extractor.extract_audio_cover_art())
-        {
-            return false;
-        }
+        extractor.save_screenshot(ofname);
+        return true;
     }
-    extractor.save_screenshot(ofname);
-    return true;
-}
+    return false;
 }
 
+}  // namespace
+
 int main(int argc, char** argv)
 {
+    char const * const progname = basename(argv[0]);
+
+    TraceMessageHandler message_handler("vs-thumb");
+
     gst_init(&argc, &argv);
-    if (argc != 3)
+
+    if (argc < 2 || argc > 3)
     {
-        fprintf(stderr, "%s <source file> <output file>\n", argv[0]);
+        fprintf(stderr, "usage: %s source-file [output-file.tiff]\n", progname);
         return 1;
     }
-    string uri;
-    string outfile(argv[2]);
+
+    string uri = command_line_arg_to_uri(argv[1]);
+    string outfile(argc == 2 ? "" : argv[2]);
+
+    // Output file name must end in .tiff.
+    if (!outfile.empty() && !boost::algorithm::ends_with(outfile, ".tiff"))
+    {
+        fprintf(stderr, "%s: invalid output file name: %s (missing .tiff extension)\n", progname, outfile.c_str());
+        return 2;
+    }
+
     bool success = false;
-
-    try
-    {
-        uri = command_line_arg_to_uri(argv[1]);
-    }
-    catch (exception const& e)
-    {
-        fprintf(stderr, "Error parsing \"%s\": %s\n", argv[1], e.what());
-        return 1;
-    }
-
     try
     {
         success = extract_thumbnail(uri, outfile);
     }
     catch (exception const& e)
     {
-        fprintf(stderr, "Error creating thumbnail: %s\n", e.what());
+        fprintf(stderr, "%s: Error creating thumbnail: %s\n", progname, e.what());
         return 2;
     }
 
